"""Request Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from ..const import ALL
from .base import BaseModel
from .const import ProtocolType
from .request_common import (
    _CommandBody,
    _Common,
    _Common3,
    _CustomFilterAttr,
    _Fields,
    _FilesystemDirectory,
    _FilesystemFolder,
    _LocalizationStrings,
    _LogRecord,
    _MetadataFields,
    _QualityInfo,
    _QualityProfileItems,
    _RecordCommon,
    _ReleaseProfilePreferred,
    _RootFolder,
    _Tag,
    _UpdateChanges,
)


class AddTypes(str, Enum):
    """Add types."""

    AUTOMATIC = "automatic"
    MANUAL = "manual"


class AllowFingerprintingType(str, Enum):
    """Allow fingerprinting type."""

    ALL_FILES = "allFiles"
    NEVER = "never"
    NEW_FILES = "newFiles"


class AuthenticationType(str, Enum):
    """Authentication type."""

    BASIC = "basic"
    FORMS = "forms"
    NONE = "none"


class Commands(str, Enum):
    """Commands."""

    APPLICATION_UPDATE = "ApplicationUpdate"
    BACKUP = "Backup"
    CHECK_HEALTH = "CheckHealth"
    CLEAN_RECYCLE_BIN = "CleanUpRecycleBin"
    CLEAR_BLOCKLIST = "ClearBlocklist"
    DELETE_LOGS = "DeleteLogFiles"
    DELETE_UPDATE_LOGS = "DeleteUpdateLogFiles"
    HOUSEKEEPING = "Housekeeping"
    IMPORT_LIST_SYNC = "ImportListSync"
    MANUAL_IMPORT = "ManualImport"
    MESSAGING_CLEANUP = "MessagingCleanup"
    REFRESH_MONITORED_DOWNLOADS = "RefreshMonitoredDownloads"
    RENAME_FILES = "RenameFiles"
    RSS_SYNC = "RssSync"


class CertificateValidationType(str, Enum):
    """Certificate validation type."""

    DISABLED = "disabled"
    DISABLED_LOCAL = "disabledForLocalAddresses"
    ENABLED = "enabled"


class CommandPriorityType(str, Enum):
    """Command priority type."""

    HIGH = "high"
    LOW = "low"
    NORMAL = "normal"


class CommandStatusType(str, Enum):
    """Command status type."""

    QUEUED = "queued"
    STARTED = "started"
    COMPLETED = "completed"
    FAILED = "failed"
    ABORTED = "aborted"
    CANCELLED = "cancelled"
    ORPHANED = "orphaned"


class CommandTriggerType(str, Enum):
    """Command trigger type."""

    MANUAL = "manual"
    SCHEDULED = "scheduled"
    UNSPECIFIED = "unspecified"


class HealthType(str, Enum):
    """Health type."""

    ERROR = "error"
    NOTICE = "notice"
    OK = "ok"
    WARNING = "warning"


class HostUpdateType(str, Enum):
    """Host update type."""

    APT = "apt"
    BUILT_IN = "builtIn"
    DOCKER = "docker"
    EXTERNAL = "external"
    SCRIPT = "script"


class ImageSize(str, Enum):
    """Image size."""

    LARGE = "large"
    MEDIUM = "medium"
    SMALL = "small"


class ImageType(str, Enum):
    """Image type."""

    BANNER = "banner"
    COVER = "cover"
    DISC = "disc"
    FANART = "fanart"
    HEADSHOT = "headshot"
    LOGO = "logo"
    POSTER = "poster"
    SCREENSHOT = "screenshot"
    UNKNOWN = "unknown"


class LogSortKeys(str, Enum):
    """Log sort keys."""

    ID = "id"
    LEVEL = "level"
    LOGGER = "logger"
    MESSAGE = "message"
    TIME = "time"


class MonitoringOptionsType(str, Enum):
    """Monitoring options type."""

    ALL = ALL
    EXISTING = "existing"
    FIRST = "first"
    FUTURE = "future"
    LATEST = "latest"
    MISSING = "missing"
    NONE = "none"
    UNKNOWN = "unknown"


class ProxyType(str, Enum):
    """Proxy type."""

    HTTP = "http"
    SOCKS4 = "socks4"
    SOCKS5 = "socks5"


class RescanAfterRefreshType(str, Enum):
    """Rescan after refresh type."""

    AFTER_MANUAL = "afterManual"
    ALWAYS = "always"
    NEVER = "never"


class SortDirection(str, Enum):
    """Sort direction type."""

    ASCENDING = "ascending"
    DEFAULT = "default"
    DESCENDING = "descending"


class StatusType(str, Enum):
    """Status type."""

    CONTINUING = "continuing"
    ENDED = "ended"


@dataclass(init=False)
class Diskspace(BaseModel):
    """Diskspace attributes."""

    freeSpace: int
    label: str
    path: str
    totalSpace: int


@dataclass(init=False)
class Logs(_RecordCommon):
    """Log attributes."""

    records: list[_LogRecord] = field(default_factory=list[_LogRecord])

    def __post_init__(self):
        """Post init."""
        self.records = [_LogRecord(record) for record in self.records]


@dataclass(init=False)
class LogFile(BaseModel):
    """Log file attributes."""

    contentsUrl: str
    downloadUrl: str
    filename: str
    id: int
    lastWriteTime: datetime


@dataclass(init=False)
class Tag(_Tag):
    """Tag attributes."""


@dataclass(init=False)
class SystemBackup(_Common3):
    """System backup attributes."""

    path: str
    time: datetime
    type: str


@dataclass(init=False)
class CustomFilter(BaseModel):
    """Custom filter attributes."""

    id: int
    filters: list[_CustomFilterAttr] | None = None
    label: str
    type: str

    def __post_init__(self):
        """Post init."""
        self.filters = [_CustomFilterAttr(filter) for filter in self.filters or []]


@dataclass(init=False)
class RootFolder(_RootFolder):
    """Root folder attributes."""


@dataclass(init=False)
class HostConfig(BaseModel):
    """Host config attributes."""

    analyticsEnabled: bool
    apiKey: str
    authenticationMethod: str
    backupFolder: str
    backupInterval: int
    backupRetention: int
    bindAddress: str
    branch: str
    certificateValidation: str
    consoleLogLevel: str
    enableSsl: bool
    id: int
    launchBrowser: bool
    logLevel: str
    password: str
    port: int
    proxyBypassFilter: str
    proxyBypassLocalAddresses: bool
    proxyEnabled: bool
    proxyHostname: str
    proxyPassword: str
    proxyPort: int
    proxyType: str
    proxyUsername: str
    sslCertHash: str
    sslCertPassword: str
    sslCertPath: str
    sslPort: int
    updateAutomatically: bool
    updateMechanism: str
    updateScriptPath: str
    urlBase: str
    username: str


@dataclass(init=False)
class UIConfig(BaseModel):
    """UI config attributes."""

    calendarWeekColumnHeader: str
    enableColorImpairedMode: bool
    firstDayOfWeek: int
    id: int
    longDateFormat: str
    movieInfoLanguage: int
    movieRuntimeFormat: str
    shortDateFormat: str
    showRelativeDates: bool
    timeFormat: str
    uiLanguage: int


@dataclass(init=False)
class SystemStatus(BaseModel):
    """System status attributes."""

    appData: str
    appName: str
    authentication: str
    branch: str
    buildTime: datetime
    instanceName: str
    isAdmin: bool
    isDebug: bool
    isDocker: bool
    isLinux: bool
    isMono: bool
    isMonoRuntime: bool
    isNetCore: bool
    isOsx: bool
    isProduction: bool
    isUserInteractive: bool
    isWindows: bool
    migrationVersion: int
    mode: str
    osName: str
    osVersion: str
    packageUpdateMechanism: str
    runtimeName: str
    runtimeVersion: str
    sqliteVersion: str
    startTime: datetime
    startupPath: str
    urlBase: str
    version: str


@dataclass(init=False)
class Command(_Common3):
    """Command attributes."""

    body: type[_CommandBody] = field(default=_CommandBody)
    commandName: str
    duration: str
    ended: datetime
    lastExecutionTime: datetime
    message: str
    priority: str
    queued: datetime
    sendUpdatesToClient: bool
    started: datetime
    stateChangeTime: datetime
    status: str
    trigger: str
    updateScheduledTask: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.body = _CommandBody(self.body)


@dataclass(init=False)
class DownloadClient(_Common):
    """Download client attributes."""

    configContract: str
    enable: bool
    id: int
    priority: int
    protocol: ProtocolType
    tags: list[int]


@dataclass(init=False)
class DownloadClientConfig(BaseModel):
    """Download client configuration attributes."""

    autoRedownloadFailed: bool
    checkForFinishedDownloadInterval: int
    downloadClientWorkingFolders: str
    enableCompletedDownloadHandling: bool
    id: int
    removeCompletedDownloads: bool
    removeFailedDownloads: bool


@dataclass(init=False)
class Filesystem(BaseModel):
    """Filesystem attributes."""

    directories: list[_FilesystemDirectory] | None = None
    files: list
    parent: str

    def __post_init__(self):
        """Post init."""
        self.directories = [_FilesystemDirectory(x) for x in self.directories or []]


@dataclass(init=False)
class Health(BaseModel):
    """Health attributes."""

    message: str
    source: str
    type: str
    wikiUrl: str


@dataclass(init=False)
class ImportListExclusion(BaseModel):
    """Import list exclusion attributes."""

    artistName: str
    authorName: str
    foreignId: str
    id: int | None = None
    movieTitle: str
    movieYear: int
    title: str
    tmdbId: int
    tvdbId: int


@dataclass(init=False)
class Indexer(_Common3):
    """Indexer attributes."""

    configContract: str
    enableAutomaticSearch: bool
    enableInteractiveSearch: bool
    enableRss: bool
    fields: list[_Fields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    priority: int
    protocol: ProtocolType
    supportsRss: bool
    supportsSearch: bool
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class IndexerConfig(BaseModel):
    """Indexer configuration attributes."""

    allowHardcodedSubs: bool
    availabilityDelay: int
    id: int
    maximumSize: int
    minimumAge: int
    preferIndexerFlags: bool
    retention: int
    rssSyncInterval: int
    whitelistedHardcodedSubs: str


@dataclass(init=False)
class Language(_Common3):
    """Language attributes."""

    nameLower: str


@dataclass(init=False)
class Localization(BaseModel):
    """Localization attributes."""

    Strings: type[_LocalizationStrings] = field(default=_LocalizationStrings)

    def __post_init__(self):
        """Post init."""
        self.Strings = _LocalizationStrings(self.Strings)


@dataclass(init=False)
class MediaManagementConfig(BaseModel):
    """Media management config attributes."""

    allowFingerprinting: str
    autoRenameFolders: bool
    autoUnmonitorPreviouslyDownloadedBooks: bool
    autoUnmonitorPreviouslyDownloadedEpisodes: bool
    autoUnmonitorPreviouslyDownloadedMovies: bool
    autoUnmonitorPreviouslyDownloadedTracks: bool
    chmodFolder: str
    chownGroup: str
    copyUsingHardlinks: bool
    createEmptyAuthorFolders: bool
    createEmptyMovieFolders: bool
    createEmptySeriesFolders: bool
    deleteEmptyFolders: bool
    downloadPropersAndRepacks: str
    enableMediaInfo: bool
    episodeTitleRequired: str
    extraFileExtensions: str
    fileDate: str
    id: int
    importExtraFiles: bool
    minimumFreeSpaceWhenImporting: int
    pathsDefaultStatic: bool
    recycleBin: str
    recycleBinCleanupDays: int
    rescanAfterRefresh: str
    setPermissionsLinux: bool
    skipFreeSpaceCheckWhenImporting: bool
    watchLibraryForChanges: bool


@dataclass(init=False)
class MetadataConfig(_Common3):
    """Metadata config attributes."""

    configContract: str
    enable: bool
    fields: list[_MetadataFields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.fields = [_MetadataFields(field) for field in self.fields or []]


@dataclass(init=False)
class QualityDefinition(BaseModel):
    """Quality definition attributes."""

    id: int
    maxSize: float
    minSize: float
    preferredSize: int
    quality: type[_QualityInfo] = field(default=_QualityInfo)
    title: str
    weight: int

    def __post_init__(self):
        """Post init."""
        self.quality = _QualityInfo(self.quality)


@dataclass(init=False)
class QualityProfile(_Common3):
    """Quality profile attributes."""

    cutoff: int
    cutoffFormatScore: int
    formatItems: list
    items: list[_QualityProfileItems] | None = None
    language: type[_Common3] = field(default=_Common3)
    minFormatScore: int
    upgradeAllowed: bool

    def __post_init__(self):
        """Post init."""
        self.items = [_QualityProfileItems(item) for item in self.items or []]
        self.language = _Common3(self.language)


@dataclass(init=False)
class QueueStatus(BaseModel):
    """Queue status attributes."""

    count: int
    errors: bool
    totalCount: int
    unknownCount: int
    unknownErrors: bool
    unknownWarnings: bool
    warnings: bool


@dataclass(init=False)
class ReleaseProfile(BaseModel):
    """Release profile attributes."""

    enabled: bool
    id: int
    ignored: str
    includePreferredWhenRenaming: bool
    indexerId: int
    preferred: list[_ReleaseProfilePreferred] | None = None
    required: str
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.preferred = [_ReleaseProfilePreferred(x) for x in self.preferred or []]


@dataclass(init=False)
class RemotePathMapping(BaseModel):
    """Remote path mapping attributes."""

    host: str
    id: int
    localPath: str
    remotePath: str


@dataclass(init=False)
class SystemTask(_Common3):
    """System task attributes."""

    interval: int
    lastDuration: str
    lastExecution: datetime
    lastStartTime: datetime
    nextExecution: datetime
    taskName: str


@dataclass(init=False)
class Update(BaseModel):
    """Update attributes."""

    branch: str
    changes: type[_UpdateChanges] = field(default=_UpdateChanges)
    fileName: str
    hash: str
    installable: bool
    installed: bool
    installedOn: datetime
    latest: bool
    releaseDate: datetime
    url: str
    version: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.changes = _UpdateChanges(self.changes)


@dataclass(init=False)
class DelayProfile(BaseModel):
    """Delay profile attributes."""

    bypassIfHighestQuality: bool
    enableTorrent: bool
    enableUsenet: bool
    id: int
    order: int
    preferredProtocol: ProtocolType
    tags: list[int]
    torrentDelay: int
    usenetDelay: int


@dataclass(init=False)
class FilesystemFolder(_FilesystemFolder):
    """Filesystem folder attributes."""



"""Radarr Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import date
from enum import Enum

from ..const import DATE, MOVIE_ID, PATH, TITLE
from .base import BaseModel
from .radarr_common import (
    _RadarrCommon,
    _RadarrCommon2,
    _RadarrCommon3,
    _RadarrCustomFormats,
    _RadarrMovieAlternateTitle,
    _RadarrMovieCommon,
    _RadarrMovieCustomFormats,
    _RadarrMovieFileMediaInfo,
    _RadarrMovieHistoryBlocklistBase,
    _RadarrMovieHistoryData,
    _RadarrNotificationMessage,
    _RadarrParsedMovieInfo,
)
from .request import Language
from .request_common import (
    _Common2,
    _Common3,
    _Common4,
    _Common5,
    _Common6,
    _Common7,
    _Common8,
    _Editor,
    _ImportListCommon,
    _ManualImport,
    _MetadataFields,
    _Notification,
    _Quality,
    _RecordCommon,
    _ReleaseCommon,
    _Rename,
    _StatusMessage,
    _TagDetails,
)


class RadarrCommands(str, Enum):
    """Radarr commands."""

    DOWNLOADED_MOVIES_SCAN = "DownloadedMoviesScan"
    MISSING_MOVIES_SEARCH = "MissingMoviesSearch"
    REFRESH_MOVIE = "RefreshMovie"
    RENAME_MOVIE = "RenameMovie"
    RESCAN_MOVIE = "RescanMovie"


class RadarrEventType(Enum):
    """Radarr event types."""

    DELETED = 6
    FAILED = 4
    GRABBED = 1
    IGNORED = 9
    IMPORTED = 3
    RENAMED = 8


class RadarrImportListActionType(str, Enum):
    """Radarr import list action types."""

    GET_PROFILES = "getProfiles"
    GET_TAGS = "getTags"


class RadarrSortKeys(str, Enum):
    """Radarr sort keys."""

    DATE = DATE
    DOWNLOAD_CLIENT = "downloadClient"
    ID = "id"
    INDEXER = "indexer"
    LANGUAGES = "languages"
    MESSAGE = "message"
    MOVIE_ID = MOVIE_ID
    MOVIE_TITLE = "movies.sortTitle"
    PATH = PATH
    PROGRESS = "progress"
    PROTOCOL = "protocol"
    QUALITY = "quality"
    RATINGS = "ratings"
    RELEASE_TITLE = TITLE
    SIZE = "size"
    SOURCE_TITLE = "sourcetitle"
    STATUS = "status"
    TIMELEFT = "timeleft"


@dataclass(init=False)
class RadarrMovieFile(_RadarrMovieCommon):
    """Radarr movie file attributes."""

    dateAdded: date
    edition: str
    indexerFlags: int
    mediaInfo: type[_RadarrMovieFileMediaInfo] = field(
        default=_RadarrMovieFileMediaInfo
    )
    movieId: int
    originalFilePath: str
    path: str
    qualityCutoffNotMet: bool
    relativePath: str
    sceneName: str
    size: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.mediaInfo = _RadarrMovieFileMediaInfo(self.mediaInfo)


@dataclass(init=False)
class RadarrMovieHistory(_RadarrMovieHistoryBlocklistBase, _Common2):
    """Radarr movie history attributes."""

    data: type[_RadarrMovieHistoryData] = field(default=_RadarrMovieHistoryData)
    qualityCutoffNotMet: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.data = _RadarrMovieHistoryData(self.data)


@dataclass(init=False)
class RadarrHistory(_RecordCommon):
    """Radarr history attributes."""

    records: list[RadarrMovieHistory] = field(default_factory=list[RadarrMovieHistory])

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.records = [RadarrMovieHistory(record) for record in self.records]


@dataclass(init=False)
class RadarrBlocklistMovie(_Common7, _RadarrMovieHistoryBlocklistBase):
    """Radarr blocklist movie attributes."""


@dataclass(init=False)
class RadarrBlocklist(_RecordCommon):
    """Radarr blocklist attributes."""

    records: list[RadarrBlocklistMovie] = field(
        default_factory=list[RadarrBlocklistMovie]
    )

    def __post_init__(self):
        """Post init."""
        self.records = [RadarrBlocklistMovie(record) for record in self.records]


@dataclass(init=False)
class RadarrQueueDetail(_Common4, _Common8):
    """Radarr queue details attributes."""

    customFormats: list[_RadarrMovieCustomFormats] | None = None
    errorMessage: str
    languages: list[_Common3] | None = None
    movieId: int

    def __post_init__(self):
        """Post init."""
        self.customFormats = [
            _RadarrMovieCustomFormats(custForm) for custForm in self.customFormats or []
        ]
        self.languages = [_Common3(language) for language in self.languages or []]
        self.statusMessages = [
            _StatusMessage(statMsg) for statMsg in self.statusMessages or []
        ]
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class RadarrQueue(_RecordCommon):
    """Radarr queue attributes."""

    records: list[RadarrQueueDetail] = field(default_factory=list[RadarrQueueDetail])

    def __post_init__(self):
        """Post init."""
        self.records = [RadarrQueueDetail(record) for record in self.records]


@dataclass(init=False)
class RadarrImportList(_ImportListCommon, _RadarrCommon, _RadarrCommon2):
    """Radarr import attributes."""

    enableAuto: bool
    enabled: bool
    listType: str
    searchOnAdd: bool
    shouldMonitor: bool
    tags: list[int]


@dataclass(init=False)
class RadarrNotification(_Common3, _Notification):
    """Radarr notification attributes."""

    fields: list[_MetadataFields] | None = None
    message: type[_RadarrNotificationMessage] = field(
        default=_RadarrNotificationMessage
    )

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.fields = [_MetadataFields(field) for field in self.fields or []]
        self.message = _RadarrNotificationMessage(self.message)


@dataclass(init=False)
class RadarrTagDetails(_TagDetails):
    """Radarr tag details attributes."""

    indexerIds: list[int]
    movieIds: list[int]


@dataclass(init=False)
class RadarrNamingConfig(BaseModel):
    """Radarr host naming config attributes."""

    colonReplacementFormat: str
    id: int
    includeQuality: bool
    movieFolderFormat: str
    renameMovies: bool
    replaceIllegalCharacters: bool
    replaceSpaces: bool
    standardMovieFormat: str


@dataclass(init=False)
class RadarrMovie(_RadarrCommon2, _RadarrCommon3, _Common6):
    """Radarr movie attributes."""

    added: date
    alternateTitles: list[_RadarrMovieAlternateTitle] | None = None
    cleanTitle: str
    folderName: str
    hasFile: bool
    isAvailable: bool
    movieFile: type[RadarrMovieFile] = field(default=RadarrMovieFile)
    originalTitle: str
    path: str
    rootFolderPath: str
    secondaryYearSourceId: int
    sizeOnDisk: int
    tags: list[int]
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.alternateTitles = [
            _RadarrMovieAlternateTitle(alternateTitle)
            for alternateTitle in self.alternateTitles or []
        ]
        self.movieFile = RadarrMovieFile(self.movieFile)


@dataclass(init=False)
class RadarrImportListMovie(_RadarrCommon3):
    """Radarr import list movie attributes."""

    folder: str
    isExcluded: bool
    isExisting: bool
    isRecommendation: bool
    lists: list[int]
    overview: str
    remotePoster: str


@dataclass(init=False)
class RadarrCalendarItem(RadarrMovie):
    """Radarr calendar attributes."""


@dataclass(init=False)
class RadarrMovieEditor(_Editor):
    """Radarr root folder attributes."""

    movieIds: list[int]


@dataclass(init=False)
class RadarrParse(BaseModel):
    """Radarr parse attributes."""

    movie: type[RadarrMovie] = field(default=RadarrMovie)
    parsedMovieInfo: type[_RadarrParsedMovieInfo] = field(
        default=_RadarrParsedMovieInfo
    )
    title: str

    def __post_init__(self):
        """Post init."""
        self.movie = RadarrMovie(self.movie)
        self.parsedMovieInfo = _RadarrParsedMovieInfo(self.parsedMovieInfo)


@dataclass(init=False)
class RadarrRelease(_ReleaseCommon):
    """Radarr release attributes."""

    customFormats: list[_RadarrCustomFormats] | None = None
    customFormatScore: int
    edition: str
    imdbId: int
    indexerFlags: list[str]
    languages: list[_Common3] | None = None
    movieTitles: list[str]
    quality: type[_Quality] = field(default=_Quality)
    releaseGroup: str
    releaseHash: str
    tmdbId: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.customFormats = [_RadarrCustomFormats(x) for x in self.customFormats or []]
        self.languages = [_Common3(x) for x in self.languages or []]
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class RadarrRename(_Rename):
    """Radarr rename attributes."""

    movieFileId: int
    movieId: int


@dataclass(init=False)
class RadarrManualImport(_ManualImport):
    """Radarr manual import attributes."""

    folderName: str
    languages: list[_Common3] | None = None
    movie: type[RadarrMovie] = field(default=RadarrMovie)
    relativePath: str
    releaseGroup: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.languages = [_Common3(x) for x in self.languages or []]
        self.movie = RadarrMovie(self.movie)


@dataclass(init=False)
class RadarrExtraFile(BaseModel):
    """Radarr extra file attributes."""

    extension: str
    id: int
    movieFileId: int
    movieId: int
    relativePath: str
    type: str


@dataclass(init=False)
class RadarrIndexerFlag(Language):
    """Radarr indexer flag attributes."""


@dataclass(init=False)
class RadarrRestriction(BaseModel):
    """Radarr restriction attributes."""

    id: int
    ignored: str
    required: str
    tags: list[int]


@dataclass(init=False)
class RadarrCredit(BaseModel):
    """Radarr credit attributes."""

    personName: str
    creditTmdbId: str
    personTmdbId: int
    movieId: int
    images: list[_Common5] | None = None
    character: str
    order: int
    type: str
    id: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_Common5(image) for image in self.images or []]


@dataclass(init=False)
class RadarrAltTitle(BaseModel):
    """Radarr alternate title attributes."""

    sourceType: str
    movieId: int
    title: str
    sourceId: int
    votes: int
    voteCount: int
    language: type[_Common3] = field(default=_Common3)
    id: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language)



"""PyArr base model."""
from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime
from enum import Enum
from typing import Any

import ciso8601

from ..const import ATTR_DATA
from .const import (
    CONVERT_TO_BOOL,
    CONVERT_TO_DATE,
    CONVERT_TO_DATETIME,
    CONVERT_TO_ENUM,
    CONVERT_TO_FLOAT,
    CONVERT_TO_INTEGER,
    ProtocolType,
)


def get_datetime(
    _input: datetime | str | None, utc: bool = False
) -> datetime | str | int | None:
    """Convert input to datetime object."""
    if isinstance(_input, str):
        if _input.isnumeric():
            return int(_input)
        if utc:
            return ciso8601.parse_datetime(_input)
        return ciso8601.parse_datetime_as_naive(_input)
    return _input


def get_date(_input: datetime | str | None) -> date | None:
    """Convert input to date object."""
    if (result := get_datetime(_input)) and isinstance(result, datetime):
        return result.date()
    return None


def get_enum_value(val: str) -> str | Enum:
    """Convert input to the correct enum."""
    for protocol in ProtocolType:
        if (
            val.isnumeric()
            and protocol.value == int(val)
            or protocol.name.lower() == val
        ):
            return protocol
    return val


def toraw(obj):
    """Convert object to dict."""
    if isinstance(obj, dict):
        return {k: toraw(v) for k, v in obj.items()}
    if hasattr(obj, "__iter__") and not isinstance(obj, str):
        return [toraw(v) for v in obj]
    if hasattr(obj, "attributes"):
        return obj.attributes
    if isinstance(obj, datetime):
        return f"{obj.isoformat()}Z"
    return obj


def generate_data(
    data: dict[str, Any] | list[dict[str, Any]], datatype: Any = None
) -> Any:
    """Generate data."""
    if datatype is None:
        return data

    if isinstance(data, list):
        return [datatype(item) for item in data]

    return datatype(data)


@dataclass(init=False)
class BaseModel:
    """BaseModel."""

    def __init__(
        self,
        data: dict[str, Any] | list[dict[str, Any]],
        datatype: Any = None,
    ) -> None:
        """Init."""
        self.basedata = None
        if isinstance(data, dict):
            for key, value in data.items():
                if key == ATTR_DATA:
                    value = generate_data(value, datatype)
                elif key in CONVERT_TO_DATETIME:
                    if key == "airDateUtc":
                        value = get_datetime(value, utc=True)
                    else:
                        value = get_datetime(value)
                elif key in CONVERT_TO_DATE:
                    value = get_date(value)
                elif key in CONVERT_TO_ENUM:
                    value = get_enum_value(value)
                elif key in CONVERT_TO_FLOAT and value is not None:
                    value = float(value)
                elif key in CONVERT_TO_INTEGER and value is not None:
                    try:
                        value = int(value)
                    except ValueError:
                        pass
                elif key in CONVERT_TO_BOOL:
                    value = False if value == "False" else bool(value)
                self.__setattr__(key, value)

        self.__post_init__()

    def __post_init__(self):
        """Post init."""

    @property
    def attributes(self):
        """Return attributes of the object."""
        return {
            k: v
            if isinstance(v, bool)
            else str(v)
            if k in CONVERT_TO_INTEGER
            else toraw(v)
            for k, v in self.__dict__.items()
            if k != "basedata"
        }
